<?php
/**
 * @package   Barn2\woocommerce-product-filters
 * @author    Barn2 Plugins <support@barn2.com>
 * @license   GPL-3.0
 * @copyright Barn2 Media Ltd
 */

namespace Barn2\Plugin\WC_Filters\Admin;

use Barn2\Plugin\WC_Filters\Dependencies\Illuminate\Support\Collection;
use Barn2\Plugin\WC_Filters\Diff;
use Barn2\Plugin\WC_Filters\Model\Filter;
use Barn2\Plugin\WC_Filters\Model\Filters\Attribute;
use Barn2\Plugin\WC_Filters\Model\Filters\Color;
use Barn2\Plugin\WC_Filters\Model\Filters\Taxonomy;
use Barn2\Plugin\WC_Filters\Model\Group;
use Barn2\Plugin\WC_Filters\Model\Index;
use Barn2\Plugin\WC_Filters\Utils\Products;
use Barn2\Plugin\WC_Filters\Utils\Settings;
use Barn2\Plugin\WC_Filters\Dependencies\Lib\Registerable;
use Barn2\Plugin\WC_Filters\Utils\Filters;

/**
 * Handles all ajax requests from the admin panel for the plugin.
 */
class Admin_Ajax implements Registerable {

	/**
	 * Hook into WP
	 *
	 * @return void
	 */
	public function register() {
		add_action( 'wp_ajax_wcf_save_filter', [ $this, 'save_filter' ] );
		add_action( 'wp_ajax_wcf_get_filters', [ $this, 'get_filters' ] );
		add_action( 'wp_ajax_wcf_delete_filter', [ $this, 'delete_filter' ] );
		add_action( 'wp_ajax_wcf_save_priority', [ $this, 'save_priority' ] );
		add_action( 'wp_ajax_wcf_get_terms_taxonomies', [ $this, 'get_terms_taxonomies' ] );
		add_action( 'wp_ajax_wcf_get_group', [ $this, 'get_group' ] );
		add_action( 'wp_ajax_wcf_save_group', [ $this, 'save_group' ] );
		add_action( 'wp_ajax_wcf_create_group', [ $this, 'create_group' ] );
		add_action( 'wp_ajax_wcf_get_groups', [ $this, 'get_groups' ] );
		add_action( 'wp_ajax_wcf_delete_group', [ $this, 'delete_group' ] );
		add_action( 'wp_ajax_wcf_duplicate_group', [ $this, 'duplicate_group' ] );
	}

	/**
	 * Send a json error back to the react app.
	 *
	 * @param string $message
	 * @param array $context Any additional context to pass to the error message.
	 * @return void
	 */
	public function send_error( string $message, $context = [] ) {
		wp_send_json_error(
			[
				'error_message' => $message,
				'context'       => $context,
			],
			403
		);
	}

	/**
	 * Create or save an existing filter.
	 *
	 * @return void
	 */
	public function save_filter() {

		check_ajax_referer( 'wcf_save_filter_nonce', 'nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			$this->send_error( esc_html__( 'You are not authorized', 'woocommerce-product-filters' ) );
		}

		$filter_name = isset( $_POST['filter_name'] ) ? stripslashes( sanitize_text_field( $_POST['filter_name'] ) ) : false;
		$filter_by   = isset( $_POST['filter_by'] ) ? sanitize_text_field( $_POST['filter_by'] ) : false;
		$editing     = isset( $_POST['editing'] ) ? absint( $_POST['editing'] ) : false;
		$groups      = isset( $_POST['groups'] ) ? wc_clean( $_POST['groups'] ) : [];
		$slug        = sanitize_title_with_dashes( $filter_name, '', 'save' );

		if ( ! empty( $groups ) && ! is_array( $groups ) ) {
			$groups = [ $groups ];
		}

		if ( ! $filter_name ) {
			$this->send_error( esc_html__( 'Please enter a name for this filter', 'woocommerce-product-filters' ) );
		}

		// We only need some values not all of them.
		$options = ( new Collection( wc_clean( $_POST ) ) )->except( [ 'filter_name', 'filter_by', 'action', 'nonce', 'editing', 'groups' ] );
		$filter  = false;

		if ( $editing ) {
			$filter = Filter::find( $editing );

			if ( ! $filter instanceof Filter ) {
				$this->send_error( esc_html__( 'Something went wrong while updating the filter', 'woocommerce-product-filters' ) );
			}

			$filter->update(
				[
					'name'      => $filter_name,
					'filter_by' => $filter_by,
					'options'   => $options->toArray(),
					'slug'      => $slug,
				]
			);

			$indexed = Index::where( 'filter_id', $filter->getID() )->update(
				[
					'facet_name' => $filter_name,
				]
			);

		} else {
			$filter = Filter::create(
				[
					'name'      => $filter_name,
					'slug'      => $slug,
					'filter_by' => $filter_by,
					'options'   => $options->toArray(),
					'priority'  => 0,
				],
				is_array( $groups ) ? $groups : []
			);
		}

		if ( $filter instanceof Filter && ! empty( $filter->getID() ) ) {
			// Update the groups to which the filter belongs to if any has been selected.
			if ( is_array( $groups ) ) {
				$filter->update_groups( $groups );
			}

			wp_send_json_success(
				[
					'reindex' => Diff::is_reindex_needed(),
				]
			);
		}

		$this->send_error( esc_html__( 'Something went wrong while creating the filter', 'woocommerce-product-filters' ) );
	}

	/**
	 * Get filters from the database ready for display in the list table.
	 * Load groups too.
	 *
	 * @return void
	 */
	public function get_filters() {

		check_ajax_referer( 'wcf_get_filters_nonce', 'nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			$this->send_error( esc_html__( 'You are not authorized', 'woocommerce-product-filters' ) );
		}

		$filters = Filter::orderBy( 'priority', 'asc' )
					->orderBy( 'id', 'desc' )
					->where( 'parent_filter', 0 )
					->get();

		// Lazy load groups into the filters.
		if ( $filters && ! $filters->isEmpty() ) {
			$filters->each(
				function ( $instance ) {
					$instance->append( 'groups' );
					$instance->append( 'groups_names' );
				}
			);
		}

		$groups = Group::orderBy( 'priority', 'asc' )
					->orderBy( 'id', 'desc' )
					->get();

		wp_send_json_success(
			[
				'filters' => $filters->toArray(),
				'groups'  => $groups->toArray(),
			]
		);
	}

	/**
	 * Save the priority of groups.
	 *
	 * @return void
	 */
	public function save_priority() {

		check_ajax_referer( 'wcf_save_priority_nonce', 'nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			$this->send_error( esc_html__( 'You are not authorized', 'woocommerce-product-filters' ) );
		}

		$data = isset( $_POST['data'] ) && ! empty( $_POST['data'] ) ? wc_clean( $_POST['data'] ) : [];

		if ( empty( $data ) || ! is_array( $data ) ) {
			$this->send_error( esc_html__( 'No records to update have been found.', 'woocommerce-product-filters' ) );
		}

		foreach ( $data as $group ) {
			$the_group = Group::find( $group['id'] );

			$the_group->update(
				[
					'priority' => absint( $group['priority'] ),
				]
			);
		}

		wp_send_json_success();
	}

	/**
	 * Delete a filter from the database.
	 *
	 * @return void
	 */
	public function delete_filter() {

		check_ajax_referer( 'wcf_delete_item_nonce', 'nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			$this->send_error( esc_html__( 'You are not authorized', 'woocommerce-product-filters' ) );
		}

		$item_id = isset( $_POST['item_id'] ) ? absint( $_POST['item_id'] ) : false;

		if ( ! $item_id ) {
			$this->send_error( esc_html__( 'No filter found matching the criteria.', 'woocommerce-product-filters' ) );
		}

		$filter = Filter::find( $item_id );

		if ( $filter ) {
			$filter->delete();
		}

		wp_send_json_success(
			[
				'reindex' => Diff::is_reindex_needed(),
			]
		);
	}

	/**
	 * Duplicate a filter group.
	 *
	 * @return void
	 */
	public function duplicate_group(): void {
		check_ajax_referer( 'wcf_duplicate_group_nonce', 'nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			$this->send_error( esc_html__( 'You are not authorized', 'woocommerce-product-filters' ) );
		}

		$item_id = isset( $_POST['item_id'] ) ? absint( $_POST['item_id'] ) : false;

		if ( ! $item_id ) {
			$this->send_error( esc_html__( 'No filter found matching the criteria.', 'woocommerce-product-filters' ) );
		}

		$group = Group::find( $item_id );

		if ( ! $group instanceof Group || empty( $group->getID() ) ) {
			$this->send_error( esc_html__( 'No filter found matching the criteria.', 'woocommerce-product-filters' ) );
		}

		$new_group = $group->replicate();
		$new_group->save();

		$filters = $group->get_filters();

		// Delete the previous filters.
		$new_group->update(
			[
				'name'     => sanitize_text_field( $new_group->name ) . ' - ' . esc_html__( 'Copy', 'woocommerce-product-filters' ),
				'filters'  => [],
				'priority' => absint( $new_group->priority + 1 ),
			]
		);

		// Add filters to the new group.
		if ( ! empty( $filters ) ) {
			foreach ( $filters as $filter ) {
				$new_filter = $filter->replicate();
				$new_filter->save();

				$new_group->add_filter( $new_filter );
			}
		}

		$this->fix_filter_ids( $new_group );
		$this->fix_parent_filter_ids( $new_group );

		wp_send_json_success(
			[
				'reindex'  => true,
				'group_id' => $new_group->getID(),
			]
		);
	}

	/**
	 * Navigate through the filters and fix the "filter_id" property
	 * inside the options array but only if "filter_id" exists.
	 *
	 * The new filter_id is the one of the newly created filter.
	 *
	 * This is needed because when duplicating a group, the filters
	 * are duplicated too but the filter_id is not updated.
	 *
	 * @param Group $group
	 * @return void
	 */
	private function fix_filter_ids( Group $group ): void {
		$filters = $group->get_filters();

		if ( ! empty( $filters ) ) {
			foreach ( $filters as $filter ) {
				$options = $filter->get_options();

				if ( isset( $options['filter_id'] ) ) {
					$options['filter_id'] = $filter->getID();

					$filter->update(
						[
							'options' => $options,
						]
					);
				}
			}
		}
	}

	/**
	 * Navigate through the filters and fix the "parent_filter" column
	 * but only if the "parent_filter" is different than 0.
	 *
	 * To retrieve the new parent_filter we need to query the database
	 * and get the new filter_id of the filter in the group that has
	 * the option "attributes_mode" set to "all".
	 *
	 * @param Group $group
	 * @return void
	 */
	private function fix_parent_filter_ids( Group $group ): void {
		// Get all the filters.
		$filters = $group->get_filters();

		// Find the filter that has the "attributes_mode" option set to "all".
		$all_atts_filter = $filters->where( 'filter_by', 'attributes' )
									->where( 'options.attributes_mode', 'all' )
									->first();

		// If we found the filter, get the new filter_id.
		if ( $all_atts_filter instanceof Filter ) {
			$new_filter_id = $all_atts_filter->getID();

			// Update the parent_filter column for all the filters where the parent_filter is different than 0.
			$filters->where( 'parent_filter', '!=', 0 )
					->each(
						function ( Filter $filter ) use ( $new_filter_id ) {
							$filter->update(
								[
									'parent_filter' => $new_filter_id,
								]
							);
						}
					);
		}
	}

	/**
	 * Get the list of taxonomies and attributes available for products.
	 *
	 * @return void
	 */
	public function get_terms_taxonomies() {

		check_ajax_referer( 'wcf_get_terms_taxonomies_nonce', 'nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			$this->send_error( esc_html__( 'You are not authorized', 'woocommerce-product-filters' ) );
		}

		$taxonomies   = Settings::parse_array_for_select_control( Products::get_registered_taxonomies( true ) );
		$empty_option = [
			[
				'value' => '',
				'label' => '',
			],
		];

		wp_send_json_success(
			[
				'taxonomies' => array_merge( $empty_option, $taxonomies ),
				'attributes' => Settings::parse_array_for_select_control( Products::get_registered_attributes() ),
			]
		);
	}

	/**
	 * Save or create a group in the database.
	 *
	 * @return void
	 */
	public function save_group() {
		check_ajax_referer( 'wcf_save_group_nonce', 'nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			$this->send_error( esc_html__( 'You are not authorized', 'woocommerce-product-filters' ) );
		}

		$group_name = isset( $_POST['name'] ) ? stripslashes( wc_clean( $_POST['name'] ) ) : false;

		if ( ! $group_name || empty( $group_name ) ) {
			$this->send_error( esc_html__( 'Please enter a name for the group.', 'woocommerce-product-filters' ) );
		}

		$group_id = isset( $_POST['group_id'] ) && ! empty( $_POST['group_id'] ) && is_numeric( $_POST['group_id'] ) ? absint( $_POST['group_id'] ) : false;

		if ( ! $group_id ) {
			$this->send_error( esc_html__( 'Something went wrong: could not update the selected group', 'woocommerce-product-filters' ) );
		}

		$filters     = isset( $_POST['filters'] ) ? wc_clean( $_POST['filters'] ) : [];
		$group       = Group::find( $group_id );
		$new_filters = $filters;

		if ( ! $group || ! $group instanceof Group ) {
			$this->send_error( esc_html__( 'Something went wrong: could not update the selected group', 'woocommerce-product-filters' ) );
		}

		$current_filters = $group->get_filters()->pluck( 'id' );

		if ( ! $filters || empty( $filters ) ) {
			$this->send_error( esc_html__( 'Please add at least 1 filter to the group.', 'woocommerce-product-filters' ) );
		}

		foreach ( $filters as $index => $filter ) {
			$this->validate_filter_settings( $filter, $index );
		}

		$filters = $this->validate_custom_fields_settings( $filters );

		$filters         = new Collection( $filters );
		$updated_filters = new Collection();

		foreach ( $filters as $index => $filter ) {
			$filter = $this->create_or_update_filter( $filter, $group_id, $index );
			$updated_filters->push( $filter );
		}

		$filters = $this->maybe_attach_children(
			$updated_filters->pluck( 'id' )
				->map( fn( $value, $key ) => (string) $value )
		);

		// Get filters that have been deleted and delete them.
		$missing = $current_filters->diff( $updated_filters->pluck( 'id' ) );

		if ( $missing->isNotEmpty() ) {
			foreach ( $missing as $missing_filter_id ) {
				$missing_filter = Filter::find( $missing_filter_id );

				// Do not delete the missing child filters.
				if ( $missing_filter instanceof Attribute && $missing_filter->has_parent() && $filters->contains( $missing_filter->parent_filter ) ) {
					continue;
				}

				if ( $missing_filter instanceof Filter ) {
					$missing_filter->delete();
				}
			}
		}

		$has_dupes = $this->has_unique_sources( $filters );

		if ( is_wp_error( $has_dupes ) ) {
			$this->send_error( $has_dupes->get_error_message() );
		}

		$group->update(
			[
				'name'    => $group_name,
				'filters' => $filters->toArray(),
			]
		);

		wp_send_json_success(
			[
				'group_id' => $group->getID(),
				'reindex'  => true,
			]
		);
	}

	/**
	 * Validate settings of custom fields.
	 *
	 * @param array $filters
	 * @return array
	 */
	public function validate_custom_fields_settings( $filters ) {

		$parsed = [];

		foreach ( $filters as $key => $filter ) {
			if ( isset( $filter['cf'] ) ) {
				$restricted = isset( $filter['cf']['restricted'] ) && $filter['cf']['restricted'] === 'true';

				if ( $restricted ) {
					$transform_to          = $filter['cf']['transform'];
					$filter['filter_type'] = $transform_to;

					unset( $filter['cf']['restricted'] );
					unset( $filter['cf']['transform'] );
				}
			}
			$parsed[] = $filter;
		}

		return $parsed;
	}

	/**
	 * When creating a group detect whether or not child filters should be
	 * attached too.
	 *
	 * How it works:
	 * Detects the "all" attribute type of filter and queries the child filters
	 * once found, push each child filter right after the parent attribute place
	 * into the array so that when the parent attribute is moved around
	 * the children follow the same exact position and respect the priority.
	 *
	 * @param Collection $filters
	 * @return Collection
	 */
	private function maybe_attach_children( Collection $filters ) {
		if ( $filters->isNotEmpty() ) {
			foreach ( $filters as $index => $filter_id ) {
				$filter = Filter::find( $filter_id );

				if ( $filter instanceof Attribute && $filter->get_attributes_mode() === 'all' ) {
					$child_filters = Filter::where( 'parent_filter', $filter->getID() )->get();

					if ( $child_filters instanceof Collection && $child_filters->isNotEmpty() ) {
						foreach ( $child_filters as $child ) {
							$filters->splice( $index + 1, 0, [ strval( $child->getID() ) ] );
						}
					}
				}
			}
		}

		return $filters;
	}

	/**
	 * Get details about a given group.
	 *
	 * @return void
	 */
	public function get_group() {

		check_ajax_referer( 'wcf_load_group_nonce', 'nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			$this->send_error( esc_html__( 'You are not authorized', 'woocommerce-product-filters' ) );
		}

		$group_id = isset( $_POST['group_id'] ) ? absint( $_POST['group_id'] ) : false;

		if ( ! $group_id ) {
			$this->send_error( esc_html__( 'No group ID has been found.', 'woocommerce-product-filters' ) );
		}

		$group = Group::find( $group_id );

		if ( ! $group instanceof Group ) {
			$this->send_error( esc_html__( 'Something went wrong, could not find the selected group.', 'woocommerce-product-filters' ) );
		}

		$filters = $group->get_filters( true );

		wp_send_json_success(
			[
				'group_id' => $group->getID(),
				'name'     => $group->name,
				'filters'  => $filters,
			]
		);
	}

	/**
	 * Create a group in the database.
	 * Used by the modal form on the add/Edit filter page.
	 *
	 * @return void
	 */
	public function create_group() {
		check_ajax_referer( 'wcf_create_group_nonce', 'nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			$this->send_error( esc_html__( 'You are not authorized', 'woocommerce-product-filters' ) );
		}

		$group_name = isset( $_POST['name'] ) ? stripslashes( wc_clean( $_POST['name'] ) ) : false;

		if ( ! $group_name || empty( $group_name ) ) {
			$this->send_error( esc_html__( 'Please enter a name for the group', 'woocommerce-product-filters' ) );
		}

		$filters = isset( $_POST['filters'] ) && is_array( $_POST['filters'] ) && ! empty( $_POST['filters'] ) ? wc_clean( $_POST['filters'] ) : [];

		if ( ! $filters || empty( $filters ) ) {
			$this->send_error( esc_html__( 'Please add at least 1 filter to the group.', 'woocommerce-product-filters' ) );
		}

		foreach ( $filters as $index => $filter ) {
			$this->validate_filter_settings( $filter, $index );
		}

		$filters = $this->validate_custom_fields_settings( $filters );

		$group = Group::create(
			[
				'name'     => $group_name,
				'priority' => 0,
				'filters'  => [],
			]
		);

		if ( ! $group instanceof Group || empty( $group->getID() ) ) {
			$this->send_error( esc_html__( 'Something went wrong while creating the group', 'woocommerce-product-filters' ) );
		}

		foreach ( $filters as $index => $filter ) {
			$filter = $this->create_or_update_filter( $filter, $group->getID(), $index );
		}

		wp_send_json_success(
			[
				'group_id' => $group->getID(),
				'reindex'  => true,
			]
		);
	}

	/**
	 * Validate various settings about a given filter.
	 *
	 * @param array $filter
	 * @param int $index
	 * @return void
	 */
	private function validate_filter_settings( array &$filter, $index ) {
		// Validate the name.
		if ( ! isset( $filter['filter_name'] ) || isset( $filter['filter_name'] ) && empty( $filter['filter_name'] ) ) {
			$this->send_error(
				esc_html__( 'One or more filters have no name assigned. Please make sure all filters have a name.', 'woocommerce-product-filters' ),
				[
					'filter_index' => $index,
				]
			);
		}

		$is_color_filter = $filter['filter_by'] === 'colors';

		if ( $is_color_filter ) {
			$selected_attribute = isset( $filter['color_attribute'] ) && ! empty( $filter['color_attribute'] ) ? sanitize_text_field( $filter['color_attribute'] ) : false;

			if ( ! $selected_attribute ) {
				$this->send_error(
					esc_html__( 'Please select a color attribute.', 'woocommerce-product-filters' ),
					[
						'filter_index' => $index,
					]
				);
			}
		}

		$is_custom_field = $filter['filter_by'] === 'cf';
		$has_cf_options  = isset( $filter['cf'] ) && is_array( $filter['cf'] ) && ! empty( $filter['cf'] );

		if ( ! $has_cf_options && $is_custom_field ) {
			$this->send_error(
				esc_html__( 'Please select a custom field.', 'woocommerce-product-filters' ),
				[
					'filter_index' => $index,
				]
			);
		}

		$is_attribute          = $filter['filter_by'] === 'attributes';
		$is_specific_attribute = isset( $filter['attributes_mode'] ) && $filter['attributes_mode'] === 'specific';

		if ( $is_attribute && $is_specific_attribute ) {
			$selected_attribute = isset( $filter['specific_attribute'] ) && ! empty( $filter['specific_attribute'] ) ? sanitize_text_field( $filter['specific_attribute'] ) : false;
			if ( ! $selected_attribute ) {
				$this->send_error(
					esc_html__( 'Please select a specific attribute.', 'woocommerce-product-filters' ),
					[
						'filter_index' => $index,
					]
				);
			}
		}
	}

	/**
	 * Create or update a filter.
	 *
	 * @param array $filter
	 * @param string|int $group_id
	 * @param int $priority
	 * @return Filter
	 */
	private function create_or_update_filter( array $filter, $group_id, $priority ) {
		$filter_name = isset( $filter['filter_name'] ) ? stripslashes( sanitize_text_field( $filter['filter_name'] ) ) : false;
		$slug        = sanitize_title_with_dashes( $filter_name, '', 'save' );
		$options     = ( new Collection( wc_clean( $filter ) ) );
		$filter_by   = isset( $filter['filter_by'] ) ? sanitize_text_field( $filter['filter_by'] ) : false;
		$filter_id   = isset( $filter['filter_id'] ) ? absint( $filter['filter_id'] ) : false;
		$slug        = preg_replace( '/[^A-Za-z0-9 ]/', '', $slug );
		$options     = ( $this->maybe_attach_taxonomy_setting( $options ) )->except( [ 'filter_name', 'filter_by' ] );
		$filter_by   = $this->maybe_adjust_filter_by( $options, $filter_by );

		if ( Filters::is_reserved_slug( $slug ) ) {
			$slug = 'p_' . $slug;
		}

		if ( $filter_id ) {
			$filter = Filter::find( $filter_id );

			$filter->update(
				[
					'name'      => $filter_name,
					'slug'      => $slug,
					'filter_by' => $filter_by,
					'options'   => $options->toArray(),
					'priority'  => $priority,
				]
			);

			Filter::maybe_regenerate_child_filters( $filter );
		} else {
			$filter = Filter::create(
				[
					'name'      => $filter_name,
					'slug'      => $slug,
					'filter_by' => $filter_by,
					'options'   => $options->toArray(),
					'priority'  => $priority,
				],
				[ $group_id ]
			);
		}

		if ( $filter instanceof Filter && ! empty( $filter->getID() ) ) {
			$filter->update_groups( [ $group_id ] );
		}

		return $filter;
	}

	/**
	 * Attach a "taxonomy" setting if the filter_by option starts with "tax_".
	 *
	 * @param Collection $options The filter options.
	 * @return Collection The filter options.
	 */
	private function maybe_attach_taxonomy_setting( Collection $options ) {
		$filter_by = $options->get( 'filter_by' ) ?? false;

		// If filter_by starts with "tax_" then it's a taxonomy.
		if ( $filter_by && strpos( $filter_by, 'tax_' ) === 0 ) {
			$taxonomy = str_replace( 'tax_', '', $filter_by );

			$options->put( 'taxonomy', $taxonomy );
		}

		return $options;
	}

	/**
	 * Adjust the filter_by option based on the filter mode.
	 *
	 * @param Collection $options The filter options.
	 * @param string $filter_by The filter_by option.
	 * @return string The filter_by option.
	 */
	private function maybe_adjust_filter_by( Collection $options, $filter_by ) {
		$filter_mode = $options->get( 'filter_method' ) ?? 'filter';

		if ( $filter_mode === 'filter' ) {
			return $filter_by;
		}

		if ( $filter_mode === 'search' ) {
			$filter_by = 'search';
		} elseif ( $filter_mode === 'sorter' ) {
			$filter_by = 'sorter';
		}

		$options->put( 'filter_mode', $filter_by );

		return $filter_by;
	}

	/**
	 * Get filter group from the database.
	 *
	 * @return void
	 */
	public function get_groups() {

		check_ajax_referer( 'wcf_get_groups_nonce', 'nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			$this->send_error( esc_html__( 'You are not authorized', 'woocommerce-product-filters' ) );
		}

		$groups = Group::orderBy( 'priority', 'asc' )
					->orderBy( 'id', 'desc' )
					->get();

		wp_send_json_success( $groups->toArray() );
	}

	/**
	 * Delete a group from the database.
	 *
	 * @return void
	 */
	public function delete_group() {

		check_ajax_referer( 'wcf_delete_group_nonce', 'nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			$this->send_error( esc_html__( 'You are not authorized', 'woocommerce-product-filters' ) );
		}

		$group_id = isset( $_POST['group_id'] ) ? absint( $_POST['group_id'] ) : false;

		if ( ! $group_id ) {
			$this->send_error( esc_html__( 'No record found matching the criteria.', 'woocommerce-product-filters' ) );
		}

		$group = Group::find( $group_id );

		if ( ! $group || ! $group instanceof Group ) {
			$this->send_error( esc_html__( 'No group found matching the criteria.', 'woocommerce-product-filters' ) );
		}

		$filters = $group->get_filters();

		if ( $filters instanceof Collection ) {
			foreach ( $filters as $filter ) {
				$filter->delete();
			}
		}

		$group->delete();

		wp_send_json_success();
	}

	/**
	 * Determine if the collection of filters has filters
	 * that are sharing the same data source or in the case
	 * of attributes, determine if there's any conflict.
	 *
	 * @param Collection $filters
	 * @return boolean|\WP_Error
	 */
	private function has_unique_sources( Collection $filters ) {
		$details = [];
		$filters = Filter::find( $filters->toArray() );

		if ( $filters ) {
			foreach ( $filters as $filter ) {
				$config = [
					'id'            => $filter->getID(),
					'name'          => $filter->name,
					'parent_filter' => $filter->parent_filter,
					'is_attribute'  => $filter instanceof Attribute || $filter instanceof Color,
					'filter_by'     => in_array( $filter->filter_by, [ 'taxonomy', 'attributes', 'colors' ], true ) ? $this->get_filter_data_source( $filter ) : $filter->filter_by,
				];

				$details[] = $config;
			}

			$collection = new Collection( $details );

			// Detect "filter_by" dupes.
			$grouped = $collection->groupBy( 'filter_by' );
			$dupes   = $grouped->filter(
				function ( Collection $groups ) {
					return $groups->count() > 1;
				}
			);

			// We don't need to check for custom fields.
			$dupes = $dupes->forget( 'cf' );

			if ( $dupes->isNotEmpty() ) {
				$dupes_names = [];

				foreach ( $dupes as $dupe ) {
					$dupes_names[] = $dupe->implode( 'name', ', ' );
				}

				return new \WP_Error(
					'dupes',
					sprintf(
						__( 'The following filters are sharing the same data source: %s. Each filter in a group must have a unique data source. Please adjust the filters accordingly.', 'woocommerce-product-filters' ),
						implode( ', ', $dupes_names )
					)
				);
			}

			// Detect attributes conflicts.
			$atts = $collection->where( 'is_attribute', true );

			// Contains "all attributes" filter and extra attribute filters.
			if ( $atts->count() > 1 && $atts->contains( 'filter_by', 'all_attributes' ) ) {
				$all_atts_filter = $atts->where( 'filter_by', 'all_attributes' )->first();

				$all_other_atts = $atts->reject(
					function ( $att ) {
						return $att['filter_by'] === 'all_attributes' || ! empty( $att['parent_filter'] );
					}
				);

				$other_atts_names = $all_other_atts->implode( 'name', ', ' );

				if ( empty( $other_atts_names ) ) {
					return true;
				}

				return new \WP_Error(
					'atts_conflict',
					sprintf(
						__( 'The "%1$s" filter has been found. Please remove the following filters from the group: %2$s.', 'woocommerce-product-filters' ),
						$all_atts_filter['name'],
						$other_atts_names
					)
				);
			}
		}

		return true;
	}

	/**
	 * Format the data source of an attribute filter.
	 * Used internally to check for dupes.
	 *
	 * @param Filter $filter
	 * @return string
	 */
	private function get_filter_data_source( Filter $filter ) {

		if ( $filter instanceof Attribute ) {
			$mode = $filter->get_option( 'attributes_mode' );

			if ( $mode === 'all' ) {
				return 'all_attributes';
			}

			return $filter->get_option( 'specific_attribute' );
		} elseif ( $filter instanceof Taxonomy ) {
			return $filter->get_taxonomy_slug();
		}

		return $filter->filter_by;
	}

	/**
	 * Determine if the group name has changed.
	 *
	 * @param int $group_id
	 * @param string $new_name
	 * @return bool
	 */
	private function did_group_name_change( $group_id, $new_name ) {
		$group = Group::find( $group_id );

		return $group->name !== $new_name;
	}

	/**
	 * Parse the group filters to an array that can be used for verification.
	 * The verification array is used to check if the group filters have changed.
	 *
	 * @param array $filters
	 * @return array
	 */
	private function parse_group_filters_to_verification_array( array $filters ) {
		$parsed = [];

		foreach ( $filters as $key => $filter ) {
			$parsed[] = [
				'filter_name' => $filter['name'],
				'filter_by'   => $filter['filter_by'],
				...$filter['options'],
			];

			$filter_id = isset( $filter['filter_id'] ) ? absint( $filter['filter_id'] ) : false;

			if ( ! $filter_id ) {
				$filter_id = isset( $filter['id'] ) ? absint( $filter['id'] ) : false;
			}

			if ( $filter_id ) {
				$parsed[ $key ]['filter_id'] = (string) $filter_id;
			}
		}

		return $parsed;
	}

	/**
	 * Check if the group filters have changed.
	 *
	 * Compare the values of the current filters with the new ones
	 * and return true if they have changed.
	 *
	 * @param array $current_filters
	 * @param array $new_filters
	 * @return bool
	 */
	private function all_keys_are_matching( array $current_filters, array $new_filters ) {
		krsort( $current_filters );
		krsort( $new_filters );

		return $current_filters !== $new_filters;
	}

	/**
	 * Check if the group filters are identical.
	 *
	 * Compare the values of the current filters with the new ones
	 * and return true if they are identical.
	 *
	 * @param array $current_filters
	 * @param array $new_filters
	 * @return bool
	 */
	private function filters_are_identical( $current_filters, $new_filters ) {
		krsort( $current_filters );
		krsort( $new_filters );

		$identical = true;

		foreach ( $new_filters as $key => $new_filter ) {
			// Find the existing filter by the filter_id.
			$existing_filter = array_filter(
				$current_filters,
				function ( $filter ) use ( $new_filter ) {
					if ( ! isset( $filter['filter_id'] ) || ! isset( $new_filter['filter_id'] ) ) {
						return false;
					}

					return $filter['filter_id'] === $new_filter['filter_id'];
				}
			);

			$existing_filter = array_shift( $existing_filter );

			if ( empty( $existing_filter ) ) {
				$identical = false;
				break;
			}

			ksort( $existing_filter );
			ksort( $new_filter );

			// Now compare all the values of each key in both arrays.
			$identical = $existing_filter === $new_filter;

			if ( ! $identical ) {
				break;
			}
		}

		return $identical;
	}

	/**
	 * Check if there were any changes to the group.
	 *
	 * @param int $group_id
	 * @param string $new_name
	 * @param array $new_filters
	 * @return bool
	 */
	private function were_there_any_group_changes( $group_id, $new_name, $new_filters ) {
		$group           = Group::find( $group_id );
		$current_filters = $this->parse_group_filters_to_verification_array( $group->get_filters()->toArray() );

		if ( $this->has_attributes_filter( $current_filters ) || $this->has_attributes_filter( $new_filters ) ) {
			return true;
		}

		$all_keys_are_matching     = $this->all_keys_are_matching( $current_filters, $new_filters );
		$all_filters_are_identical = $this->filters_are_identical( $current_filters, $new_filters );

		// Check that the count of filters in both arrays is the same.
		if ( count( $current_filters ) !== count( $new_filters ) ) {
			return true;
		}

		return $this->did_group_name_change( $group_id, $new_name ) || ! $all_keys_are_matching || ! $all_filters_are_identical;
	}

	/**
	 * Check if the array of filters has an attribute filter.
	 * The structure of the array is parsed via
	 * the parse_group_filters_to_verification_array() method.
	 *
	 * We check if the "filter_by" key is "attributes" and if
	 * a "attributes_mode" option is set to "all".
	 *
	 * @param array $filters
	 * @return boolean
	 */
	private function has_attributes_filter( array $filters ) {
		$attributes_filter = array_filter(
			$filters,
			function ( $filter ) {
				return $filter['filter_by'] === 'attributes' && $filter['attributes_mode'] === 'all';
			}
		);

		return ! empty( $attributes_filter );
	}
}
